from .deprecated import deprecated
from .logging import log_entry_exit
from .retry import retry
from .silent import silent
from .timeit import timeit
from .with_lock import with_lock

__all__ = ["deprecated", "log_entry_exit", "retry", "silent", "timeit", "with_lock"]



import shutil

import pytest

from mooch.validators import command

fake_valid_commands = ["python", "ls", "echo"]

@pytest.fixture()
def setup_fake_which(monkeypatch):
    def fake_which(cmd):
        return f"/usr/bin/{cmd}" if cmd in fake_valid_commands else None
    
    monkeypatch.setattr(shutil, "which", fake_which)

def test_check_all_commands_exist_list(setup_fake_which):
    command.check(fake_valid_commands)


def test_check_some_commands_missing(setup_fake_which):
    with pytest.raises(RuntimeError) as excinfo:
        command.check(["python", "ls", "foobar", "baz"])
    assert "foobar" in str(excinfo.value)
    assert "baz" in str(excinfo.value)


def test_check_no_commands(setup_fake_which):
    command.check([])



def test_check_all_commands_missing(setup_fake_which):
    with pytest.raises(RuntimeError) as excinfo:
        command.check(["foo", "bar"])
    assert "foo" in str(excinfo.value)
    assert "bar" in str(excinfo.value)


def test_check_variadic_args(setup_fake_which):
    command.check("python", "ls", "echo")

def test_check_mixed_args(setup_fake_which):
    command.check(["python", "ls"], "echo")
    command.check("python", ["ls"], "echo")

def test_check_variadic_args_with_missing(setup_fake_which):
    with pytest.raises(RuntimeError) as excinfo:
        command.check(["python", "ls", "foobar", "baz"])
    assert "foobar" in str(excinfo.value)
    assert "baz" in str(excinfo.value)



from mooch.validators import architecture, command, env_var, operating_system, python_version, virtual_env

command.check(["git", "python3", "curl"])
print("All required commands are available.")

architecture.check(["x86_64", "arm64"])
print("Architecture is supported.")

virtual_env.check()
print("Virtual environment is activated.")

python_version.check("3.8")
print("Python version is compatible.")

operating_system.check(["Linux", "Darwin", "Windows"])
print("Operating system is supported.")

env_var.check(["HOME"])
print("Environment variable HOME is set.")



import sys


def check() -> None:
    """Check if the current Python interpreter is running inside a virtual environment.

    Raises:
        RuntimeError: If a virtual environment is not activated.

    """
    result = hasattr(sys, "real_prefix") or (hasattr(sys, "base_prefix") and sys.base_prefix != sys.prefix)
    if not result:
        msg = "Virtual environment is not activated."
        raise RuntimeError(msg)



import logging
import time

from mooch.decorators import timeit

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


@timeit
def random_function(arg1, arg2):
    print(arg1)
    print(arg2)
    time.sleep(1.5)


if __name__ == "__main__":
    random_function("Hello", "World")



import sys

import pytest

from mooch.validators.virtual_env import check


def test_check_virtualenv_real_prefix(monkeypatch):
    # Simulate sys.real_prefix exists (old virtualenv)
    monkeypatch.setattr(sys, "real_prefix", "/some/path", raising=False)
    # base_prefix and prefix should not matter in this case
    check()  # Should not raise


def test_check_virtualenv_base_prefix_diff(monkeypatch):
    # Simulate sys.base_prefix != sys.prefix (venv)
    monkeypatch.setattr(sys, "base_prefix", "/base", raising=False)
    monkeypatch.setattr(sys, "prefix", "/venv", raising=False)
    # Remove real_prefix if present
    if hasattr(sys, "real_prefix"):
        monkeypatch.delattr(sys, "real_prefix", raising=False)
    check()  # Should not raise


def test_check_virtualenv_not_activated(monkeypatch):
    # Simulate no real_prefix and base_prefix == prefix
    if hasattr(sys, "real_prefix"):
        monkeypatch.delattr(sys, "real_prefix", raising=False)
    monkeypatch.setattr(sys, "base_prefix", "/same", raising=False)
    monkeypatch.setattr(sys, "prefix", "/same", raising=False)
    with pytest.raises(RuntimeError, match="Virtual environment is not activated."):
        check()



import asyncio
import functools
import logging
import time

logger = logging.getLogger(__name__)


def timeit(func: callable):  # noqa: ANN201
    """Log the execution time of sync or async function."""
    logger = logging.getLogger(func.__module__)

    @functools.wraps(func)
    def sync_wrapper(*args, **kwargs):  # noqa: ANN002, ANN003, ANN202
        start = time.perf_counter()
        result = func(*args, **kwargs)
        end = time.perf_counter()
        duration = end - start
        logger.debug(f"{func.__name__} executed in {duration:.6f} seconds.")
        return result

    @functools.wraps(func)
    async def async_wrapper(*args, **kwargs):  # noqa: ANN002, ANN003, ANN202
        start = time.perf_counter()
        result = await func(*args, **kwargs)
        end = time.perf_counter()
        duration = end - start
        logger.debug(f"{func.__name__} executed in {duration:.6f} seconds.")
        return result

    return async_wrapper if asyncio.iscoroutinefunction(func) else sync_wrapper



from barnone import ColoredProgressBar, ProgressBar
from zettings import Settings

from mooch.location.location import Location

__all__ = ["ColoredProgressBar", "Location", "ProgressBar", "Settings"]



import time

from mooch import ColoredProgressBar

# Basic Example
pb = ColoredProgressBar(100)
for _ in range(100):
    time.sleep(0.1)
    pb.update()

# Finish Early
pb = ColoredProgressBar(100)
for i in range(100):
    time.sleep(0.1)
    pb.update()
    if i == 50:
        pb.finish()
        break

# Delayed Start
pb = ColoredProgressBar(100, auto_start=False)
time.sleep(1)  # Simulate some work before starting
pb.start()
for _ in range(100):
    time.sleep(0.1)
    pb.update()

# Overrun Example
pb = ColoredProgressBar(5)
for _ in range(5):  # Intentionally overrun
    time.sleep(0.1)
    pb.update()
pb.update()  # This will trigger the overrun warning



import asyncio

from mooch.decorators import silent


def test_silent_suppresses_exception_and_returns_none():
    @silent()
    def raise_error():
        raise ValueError("fail")

    assert raise_error() is None


def test_silent_suppresses_exception_and_returns_fallback():
    @silent(fallback="fallback_value")
    def raise_error():
        raise RuntimeError("fail")

    assert raise_error() == "fallback_value"


def test_silent_returns_function_result_when_no_exception():
    @silent(fallback="fallback_value")
    def add(a, b):
        return a + b

    assert add(2, 3) == 5


def test_silent_passes_args_and_kwargs():
    @silent(fallback=0)
    def multiply(a, b=1):
        return a * b

    assert multiply(4, b=5) == 20


def test_silent_preserves_function_metadata():
    @silent()
    def sample_func():
        """Docstring here."""
        return 42

    assert sample_func.__name__ == "sample_func"
    assert sample_func.__doc__ == "Docstring here."


def test_silent_logs_exception_when_log_exceptions_is_true(caplog):
    @silent(log_exceptions=True)
    def raise_error():
        raise ValueError("fail")

    with caplog.at_level("ERROR"):
        result = raise_error()

    assert result is None
    assert "Silent Exception in raise_error. Returning fallback value 'None'." in caplog.text
    assert "ValueError: fail" in caplog.text


def test_silent_does_not_log_exception_when_log_exceptions_is_false(caplog):
    @silent(log_exceptions=False)
    def raise_error():
        raise ValueError("fail")

    result = raise_error()

    assert result is None
    assert "Silent Exception in raise_error. Returning fallback value 'None'." not in caplog.text
    assert "ValueError: fail" not in caplog.text


def test_silent_async_function():
    @silent(fallback="async_fallback")
    async def async_raise_error():
        await asyncio.sleep(0.01)
        raise ValueError("async fail")

    async def main():
        result = await async_raise_error()
        assert result == "async_fallback"

    asyncio.run(main())


def test_silent_async_function_success():
    @silent(fallback="async_fallback")
    async def async_add(a, b):
        await asyncio.sleep(0.01)
        return a + b

    async def main():
        result = await async_add(2, 3)
        assert result == 5

    asyncio.run(main())


def test_silent_async_function_with_args():
    @silent(fallback="async_fallback")
    async def async_multiply(a, b=1):
        await asyncio.sleep(0.01)
        return a * b

    async def main():
        result = await async_multiply(4, b=5)
        assert result == 20

    asyncio.run(main())


def test_silent_async_function_preserves_metadata():
    @silent()
    async def async_sample_func():
        """Async docstring here."""
        return 42

    assert async_sample_func.__name__ == "async_sample_func"
    assert async_sample_func.__doc__ == "Async docstring here."


def test_silent_async_function_logs_exception_when_log_exceptions_is_true(caplog):
    @silent(log_exceptions=True)
    async def async_raise_error():
        await asyncio.sleep(0.01)
        raise ValueError("async fail")

    with caplog.at_level("ERROR"):
        result = asyncio.run(async_raise_error())
        assert result is None

    assert "ValueError: async fail" in caplog.text


def test_silent_async_function_does_not_log_exception_when_log_exceptions_is_false(caplog):
    @silent(log_exceptions=False)
    async def async_raise_error():
        await asyncio.sleep(0.01)
        raise ValueError("async fail")

    result = asyncio.run(async_raise_error())
    assert result is None
    assert "Silent Exception in async_raise_error. Returning fallback value 'async_fallback'." not in caplog.text
    assert "ValueError: async fail" not in caplog.text



from __future__ import annotations

import asyncio
import functools
import threading


def with_lock(lock: threading.Lock | None = None):  # noqa: ANN201
    """Ensure a function is executed with provided Lock object.

    Notes:
        - If no lock is provided, a new Lock instance is created.
        - Caution: The created lock is only shared among the same decorated function.

    """

    def decorator(func: callable) -> callable:
        nonlocal lock
        if lock is None:
            lock = asyncio.Lock() if asyncio.iscoroutinefunction(func) else threading.Lock()

        @functools.wraps(func)
        def sync_wrapper(*args, **kwargs):  # noqa: ANN002, ANN003, ANN202
            with lock:
                return func(*args, **kwargs)

        @functools.wraps(func)
        async def async_wrapper(*args, **kwargs):  # noqa: ANN002, ANN003, ANN202
            async with lock:
                return await func(*args, **kwargs)

        return async_wrapper if asyncio.iscoroutinefunction(func) else sync_wrapper

    return decorator



import asyncio
import logging

import pytest

from mooch.decorators.timeit import timeit


def test_timeit_sync_logs_execution_time(caplog):
    @timeit
    def add(a, b):
        return a + b

    with caplog.at_level(logging.DEBUG):
        result = add(2, 3)
    assert result == 5
    assert any("add executed in" in message for message in caplog.messages)


@pytest.mark.asyncio
async def test_timeit_async_logs_execution_time(caplog):
    @timeit
    async def async_add(a, b):
        await asyncio.sleep(0.01)
        return a + b

    with caplog.at_level(logging.DEBUG):
        result = await async_add(4, 6)
    assert result == 10
    assert any("async_add executed in" in message for message in caplog.messages)


def test_timeit_preserves_function_metadata():
    @timeit
    def foo(x):
        """Test docstring."""
        return x

    assert foo.__name__ == "foo"
    assert foo.__doc__ == "Test docstring."


@pytest.mark.asyncio
async def test_timeit_async_preserves_function_metadata():
    @timeit
    async def bar(y):
        """Async docstring."""
        return y

    assert bar.__name__ == "bar"
    assert bar.__doc__ == "Async docstring."



import asyncio
import threading
import time

import pytest

from mooch.decorators import with_lock


def test_sync_lock_with_explicit_lock():
    _lock = threading.Lock()
    call_order = []

    @with_lock(_lock)
    def critical_section(x):
        call_order.append(x)
        return x * 2

    @with_lock(_lock)
    def another_critical_section(x):
        call_order.append(x)
        time.sleep(0.01)
        return x + 100

    results = []
    results2 = []

    def worker(val):
        results.append(critical_section(val))

    def worker2(val):
        results2.append(another_critical_section(val))

    threads = [threading.Thread(target=worker, args=(i,)) for i in range(5)]
    threads2 = [threading.Thread(target=worker2, args=(i,)) for i in range(100, 105)]
    for t in threads:
        t.start()
    for t2 in threads2:
        t2.start()
    for t in threads:
        t.join()
    for t in threads + threads2:
        t.join()

    assert results == [i * 2 for i in range(5)]
    assert call_order == [0, 1, 2, 3, 4, 100, 101, 102, 103, 104]


def test_sync_lock_with_no_lock_shares_lock_with_same_function_only():
    results = []

    @with_lock()
    def critical_section(x):
        for i in range(5):
            time.sleep(0.01)
            result = i + x
            results.append(result)

    def worker():
        critical_section(100)

    def worker2():
        critical_section(200)

    threads = threading.Thread(target=worker)
    threads2 = threading.Thread(target=worker2)

    threads.start()
    threads2.start()
    threads.join()
    threads2.join()

    assert results == [i + 100 for i in range(5)] + [i + 0 for i in range(200, 205)]


@pytest.mark.asyncio
async def test_async_lock_with_explicit_lock():
    _lock = asyncio.Lock()
    call_order = []

    @with_lock(_lock)
    async def async_critical_section(x):
        call_order.append(x)
        await asyncio.sleep(0.01)
        return x * 3

    async def worker(val, results):
        res = await async_critical_section(val)
        results.append(res)

    results = []
    await asyncio.gather(*(worker(i, results) for i in range(5)))
    assert results == [i * 3 for i in range(5)]
    assert call_order == [0, 1, 2, 3, 4]


@pytest.mark.asyncio
async def test_async_lock_with_default_lock():
    call_count = []

    @with_lock()
    async def increment(x):
        call_count.append(x)
        await asyncio.sleep(0.01)
        return x + 10

    result1 = await increment(5)
    result2 = await increment(7)
    assert result1 == 15
    assert result2 == 17
    assert call_count == [5, 7]
